import os
import unittest

from datetime import datetime

from stanley import Compile
from stanley import ContentParseException
from stanley import Post
from stanley import Page
from stanley import Posts


class TestPost(unittest.TestCase):

    def testMagicGetter(self):
	post = Post(src_dir='/ths/is/src', src_file='2012-12-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content')
	self.assertEqual(post.one, 'foo')

    def testIsPost(self):
	self.assertTrue(Post.is_post_file('2012-12-12-a-filename.md'))
	self.assertTrue(Post.is_post_file('2012-12-12-a-filename-2012-11-10.md'))
	self.assertTrue(Post.is_post_file('1998-12-12-a.md'))

	self.assertFalse(Post.is_post_file('202-12-12-a-filename.md'))
	self.assertFalse(Post.is_post_file('202-12-12-a-filename.txt'))
	self.assertFalse(Post.is_post_file('202-12-12-a-filename.html'))
	self.assertFalse(Post.is_post_file('202-12-12filename.txt'))
	self.assertFalse(Post.is_post_file('202-12-12--filename.md'))
	self.assertFalse(Post.is_post_file('202-12-12_-filename.md'))

    def testProcessFilename(self):
	post = Post('cats/blog', '2012-12-01-this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	post.process_filename()
	self.assertEqual(post.slug, 'this-is-the-filename')
	self.assertEqual(post.filename, 'this-is-the-filename.html')
	self.assertEqual(post.publish_date,  datetime(2012, 12, 1, 0, 0))
	self.assertEqual(post.ext, '.md')
	self.assertEqual(post.content, '<p><em>mdcontent</em></p>')
	self.assertFalse(post.draft)

    def testPermalink(self):
	post = Post('/cats/blog', '2012-12-01-this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	self.assertEqual(post.permalink, '/cats/blog/this-is-the-filename')
	post = Post('cats/blog', '2012-12-01-this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	self.assertEqual(post.permalink, '/cats/blog/this-is-the-filename')

    def testTojinja2(self):
	post = Post(src_dir='/ths/is/src', src_file='2012-12-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content')
	self.assertEqual(post.tojinja2(), {
	    'category': '/ths/is/src',
	    'content': u'<p>This is content</p>',
	    'ext': '.md',
	    'filename': 'myfile.html',
	    'one': 'foo',
	    'permalink': '/ths/is/src/myfile',
	    'publish_date': datetime(2012, 12, 12, 0, 0),
	    'slug': 'myfile',
	    'template': 'test.html',
	    'two': 'bar'})


class TestPage(unittest.TestCase):

    def testMagicGetter(self):
	page = Page(src_dir='/ths/is/src', src_file='myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content')
	self.assertEqual(page.one, 'foo')

    def testProcessFilename(self):
	page = Page('cats/blog', 'this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	page.process_filename()
	self.assertEqual(page.slug, 'this-is-the-filename')
	self.assertEqual(page.filename, 'this-is-the-filename.html')
	self.assertEqual(page.ext, '.md')
	self.assertEqual(page.content, '<p><em>mdcontent</em></p>')
	self.assertFalse(page.draft)

    def testPermalink(self):
	page = Page('/cats/blog', 'this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	self.assertEqual(page.permalink, '/cats/blog/this-is-the-filename')
	page = Page('cats/blog', 'this-is-the-filename.md', {'template': 'test.html'}, '_mdcontent_')
	self.assertEqual(page.permalink, '/cats/blog/this-is-the-filename')

    def testTojinja2(self):
	page = Page(src_dir='/ths/is/src', src_file='myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content')
	self.assertEqual(page.tojinja2(), {
	    'content': u'<p>This is content</p>',
	    'ext': '.md',
	    'filename': 'myfile.html',
	    'one': 'foo',
	    'permalink': '/ths/is/src/myfile',
	    'slug': 'myfile',
	    'template': 'test.html',
	    'two': 'bar'})


class TestPosts(unittest.TestCase):

    def testFilters(self):
	posts_ = []
	posts_.append(Post(src_dir='/blog', src_file='2012-12-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar', 'draft': True}, content='This is content'))
	posts_.append(Post(src_dir='/blog/food', src_file='2012-12-21-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content'))
	posts_.append(Post(src_dir='', src_file='2012-10-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content'))
	posts_.append(Post(src_dir='/cats/dogs/humans', src_file='2012-09-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content'))
	posts_.append(Post(src_dir='/blog', src_file='2012-08-12-myfile.md', data={'template': 'test.html', 'one': 'foo', 'two': 'bar'}, content='This is content'))

	posts = Posts(posts_)
	self.assertEqual(posts.nodraft().result, [posts_[1], posts_[2], posts_[3], posts_[4]])
	self.assertEqual(posts.category('/blog').result, [posts_[0], posts_[4]])
	self.assertEqual(posts.limit(2).result, [posts_[0], posts_[1]])
	self.assertEqual(posts.limit(4).result, [posts_[0], posts_[1], posts_[2], posts_[3]])
	self.assertEqual(posts.limit(4).result, [posts_[0], posts_[1], posts_[2], posts_[3]])
	self.assertEqual(posts.orderby('publish_date', True).result, [posts_[1], posts_[0], posts_[2], posts_[3], posts_[4]])



import os
import re

import markdown

from datetime import datetime


class ContentDataAttributeError(AttributeError):
    pass


class Content:
    """

    Parent class for content types i.e. Page & Post

    """

    POSTSPLIT = '^([0-9]{4}-[0-1][0-9]-[0-3][0-9])-+([a-zA-Z]+.*)(\.md)$'
    POSTMATCH = '^[0-9]{4}-[0-1][0-9]-[0-3][0-9]-+[a-zA-Z]+.*\.md$'

    template = None
    src_dir = None
    src_file = None
    content = None
    slug = None
    filename = None
    ext = None
    permalink = None
    draft = False
    data = None

    def __init__(self, src_dir='', src_file='', data={}, content=''):
	""" The src_dir argument is the path to the filename without the
	original source. """
	if 'template' not in data:
	    msg = '"template" is required in "%s"' % src_file
	    raise ContentDataAttributeError(msg)

	self.template = data.get('template')
	self.src_file = src_file
	self.src_dir = src_dir
	self.data = data
	self.process_filename()
	self.content = markdown.markdown(content)
	if 'draft' in data and data['draft'] is True:
	    self.draft = True
	self.permalink = self.set_permalink()

    def tojinja2(self):
        """ Prepare attributes to be passed to a Jinja template """
        filterlam = lambda (k, v): k not in ['data', 'src_dir', 'src_file']
        attributes = dict(filter(filterlam, self.__dict__.items()))
        attributes.update(self.data)
        return attributes

    def set_permalink(self):
        link = os.path.join(self.src_dir, self.slug)
        if link[0] != '/':
            return'/' + link
        return link

    @staticmethod
    def is_post_file(src):
        return re.match(Content.POSTMATCH, os.path.basename(src)) is not None

    def __getattr__(self, name, *args):
        """ Enable acess to attributes in data as if they were attributes of
        of this object. This makes templateing easier and ensures that data
        attributes from the front matter take precedence. """
        if name in self.data:
            return self.data.get(name)
        elif name in self.__dict__:
            return self.__dict__.get(name)

        raise AttributeError('Post has no attribute %s' % name)


class Post(Content):
    """

    Class that represents an individual post, als defines constants for
    patttern mathcing and spitting post filenames, and accessing post
    attributes

    """

    category = None
    publish_date = None

    def __init__(self, src_dir='', src_file='', data={}, content=''):
	""" The src_dir argument is the path to the filename without the
	original source. """
	Content.__init__(self, src_dir, src_file, data, content)
	self.category = self.src_dir

    def process_filename(self):
        """ Get publish_date, slug and dst filename for a post """
        splits = re.split(Content.POSTSPLIT, self.src_file)
        self.publish_date = datetime.strptime(splits[1], '%Y-%m-%d')
        self.slug = splits[2]
        self.filename = self.slug + '.html'
        self.ext = splits[3]


class Page(Content):
    """

    Represents a static page (i.e. not a post)

    """

    template = None
    src_dir = None
    src_file = None
    content = None
    slug = None
    filename = None
    ext = None
    permalink = None
    draft = False
    data = None

    def process_filename(self):
        """ slug and dst filename for a post """
        self.slug, self.ext = os.path.splitext(self.src_file)
        self.filename = self.slug + '.html'


class Posts:
    """

    Holds array of post objects and defines methods to filter and return
    subsets. This drives the listing of posts in the templates.

    Usage:
    ---------
    {% for post in posts.bycategory('blog').limit(5).result %}
    ... template block ...
    {% endfor %}

    """

    result = []

    def __init__(self, posts):
        self.result = posts

    def category(self, category):
        filter_ = lambda x: x.category == category
        return Posts(filter(filter_, self.result))

    def orderby(self, field, reverse):
        key = lambda k: getattr(k, field)
        return Posts(sorted(self.result, key=key, reverse=reverse))

    def limit(self, max):
	if len(self.result) < 1:
	    return Posts([])
	return Posts(self.result[0:max])

    def nodraft(self):
	filter_ = lambda x: x.draft is False
	return Posts(filter(filter_, self.result))



import os
import unittest
import shutil

from datetime import datetime

from stanley import Compile
from stanley import ContentParseException
from stanley import Post
from stanley import Page
from stanley import Posts


class TestCompile(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        self.fixtures_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'fixtures/compileTest')  # use fixtures dir as project root
        self.comp = Compile(os.path.join(self.fixtures_dir, 'config.yml'))
        self.dst_dir = os.path.join(self.fixtures_dir, 'site')
        self.src_dir = os.path.join(self.fixtures_dir, 'content')
        self.static = os.path.join(self.dst_dir, 'static')
        for root, dirs, files in os.walk(self.dst_dir):
            for f in files:
                os.unlink(os.path.join(root, f))
            for d in dirs:
                shutil.rmtree(os.path.join(root, d))

    def testGetAllFiles(self):
        files = self.comp.get_all_files()
        expected = [
            os.path.join(self.src_dir, '2012-03-12-another-test-file.md'),
            os.path.join(self.src_dir, '2012-12-11-this-is-a-test-file.md'),
            os.path.join(self.src_dir, 'index.md'),
            os.path.join(self.src_dir, 'blog/index.md'),
            os.path.join(self.src_dir, 'blog/blog_sub/index.md')]

        self.assertEqual(files, expected)

    def testParseFileContent(self):
        expectedNoContent = {
            'content': '',
            'data': {
            'permalink': 'http://atest.com',
            'title': 'This is the title',
            'template': 'test.html'}
        }
        expectedContent = {
            'content': 'This is some content',
            'data': {'permalink': 'http://atest.com',
            'title': 'This is the title',
            'template': 'test.html'}
        }
        with open(os.path.join(self.src_dir, '2012-12-11-this-is-a-test-file.md'), 'r') as f1:
            self.assertEqual(self.comp.parse_file_content(f1.read()), expectedNoContent)

        with open(os.path.join(self.src_dir, '2012-03-12-another-test-file.md'), 'r') as f2:
            self.assertEqual(self.comp.parse_file_content(f2.read()), expectedContent)

    def testGenerateSite(self):
        self.comp.generate_site()

	# posts generated properly
	for post in self.comp.jinja2env.globals.get('posts').result:
	    self.assertTrue(os.path.exists(os.path.join(self.fixtures_dir, 'site', post.src_dir, post.filename)))

	# static files generated
	self.assertTrue(os.path.exists(os.path.join(self.fixtures_dir, 'site/index.html')))

	# no unexpected files
	expected = []
	for root, dirs, files in os.walk(self.dst_dir):
	    for f in files:
		expected.append(os.path.join(root, f))

	self.assertEqual(7, len(expected), "5 + 2 css/js files")

        self.assertTrue(os.path.exists(self.static))
        self.assertTrue(os.path.exists(os.path.join(self.static, 'css/nice.css')))
        self.assertTrue(os.path.exists(os.path.join(self.static, 'js/nice.js')))


class TestCompileExceptions(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        self.fixtures_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'fixtures/exceptionsTest')  # use fixtures dir as project root
        self.comp = Compile(os.path.join(self.fixtures_dir, 'config.yml'))
        self.dst_dir = os.path.join(self.fixtures_dir, 'site')
        self.src_dir = os.path.join(self.fixtures_dir, 'content')
        for root, dirs, files in os.walk(self.dst_dir):
            for f in files:
                os.unlink(os.path.join(root, f))
            for d in dirs:
                shutil.rmtree(os.path.join(root, d))

    def testDataFrom(self):
        with self.assertRaises(ContentParseException):
            self.comp.parse_file_content('asfasfJUNK8789')



import os.path
from setuptools import setup, find_packages


install_requires = ["Flask", "PyYAML", "Markdown", "Flask-Script"]

base_dir = os.path.dirname(os.path.abspath(__file__))

setup(
    name="stanley",
    version="1.4.5",
    description="Flat file blog tool bult on Jinja2 templates",
    long_description="\n\n" + open(os.path.join(base_dir, "README.rst"), "r").read(),
    url="https://github.com/glenswinfield/stanley",
    author="Glen Swinfield",
    author_email="glen.swinfield@gmail.com",
    packages=find_packages(),
    zip_safe=False,
    install_requires=install_requires,
    test_suite="tests.get_tests",
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ]
)



# -*- coding: utf-8 -*-
"""
    stanley
    ~~~~~~~~~
    Flat file cms/blog tool

    :copyright: (c) 2013 by Glen Swinfield <glen.swinfield@gmail.com>

"""

import os
import yaml
import shutil
from jinja2 import Environment, ChoiceLoader, FileSystemLoader

from stanley import Page, Post, Posts


class TemplateException(Exception):
    """
    Thrown when template cannot be found
    """
    pass


class NoTemplateSpecified(TemplateException):
    """
    Thrown when a content file has no template specified in it's front_matter
    """
    pass


class ContentException(Exception):
    pass


class ContentParseException(ContentException):
    pass


class Compile:
    """

    Methods for taking a project directory and compiling into flat html

    """

    rootdir = None
    sourcedir = None
    destdir = None
    staticdir = None
    jinja2env = None
    config = None

    def __init__(self, config_file):
        self.config_file = config_file
        self.rootdir = project_root = os.path.dirname(config_file)
        self.sourcedir = os.path.join(project_root, 'content')
        self.destdir = os.path.join(project_root, 'site')
        self.staticdir = os.path.join(project_root, 'static')
        self.config = self.set_config()
        self.set_jinja2env()
        self.jinja2env.globals.update(self.config)

    def set_jinja2env(self):
        """ Create Jinja2 Environment with template loaded """
        self.jinja2env = Environment(loader=ChoiceLoader([
            FileSystemLoader(os.path.join(self.rootdir, 'templates'))]))

    def set_config(self):
        """ Parse YAML config file """
        with open(self.config_file, 'r') as config:
	    config = yaml.load(config.read())
	    if config is not None:
		return config
	    else:
		return {}

    def parse_file_content(self, filecontent):
        """ Get front_matter and content from a post file content """
        blocks = filecontent.split('---')
        if len(blocks) is not 3:
            raise ContentParseException("Content file could not be parsed")
        front_matter = yaml.load(blocks[1])
        return {'data': front_matter, 'content': blocks[2].strip()}

    def create_post(self, postpath):
        """ Create a Post object from a post file. """
        src_dir = os.path.dirname(postpath).replace(self.sourcedir, '')
        src_file = os.path.basename(postpath)
        with open(postpath, 'r') as f:
            contents = self.parse_file_content(f.read())
        return Post(
            src_dir=src_dir,
            src_file=src_file,
            data=contents.get('data'),
            content=contents.get('content'))

    def create_page(self, pagepath):
        """ Create a Page object from a post file. """
        src_dir = os.path.dirname(pagepath).replace(self.sourcedir, '')
        src_file = os.path.basename(pagepath)
        with open(pagepath, 'r') as f:
            contents = self.parse_file_content(f.read())
        return Page(
            src_dir=src_dir,
            src_file=src_file,
            data=contents.get('data'),
            content=contents.get('content'))

    def prepare(self):
        """ Clears out the output directory ready for the site to be
        re-compiled into it """
        for root, dirs, files in os.walk(self.destdir):
            for f in files:
                os.unlink(os.path.join(root, f))
            for d in dirs:
                shutil.rmtree(os.path.join(root, d))

    def get_all_files(self):
        """ Get a list of all of the files in the source directory. """
        filelist = []
        for root, sub_folders, files in os.walk(self.sourcedir):
            for filename in files:
                filelist.append(os.path.join(root, filename))
        return filelist

    def generate_site(self):
        """ Compile the flat html site and copy static files. """
        self.prepare()

        is_post = lambda x: Post.is_post_file(x)
        posts = map(self.create_post, filter(is_post, self.get_all_files()))

        is_page = lambda x: not Post.is_post_file(x)
        pages = map(self.create_page, filter(is_page, self.get_all_files()))

        self.jinja2env.globals.update({
            'posts': Posts(posts)
        })

        for item in posts + pages:
            if len(item.src_dir) > 0 and item.src_dir[0] == '/':
                src_dir = item.src_dir[1:]
            else:
                src_dir = item.src_dir
            dst_file = os.path.join(self.destdir, src_dir, item.filename)

            if not os.path.exists(os.path.dirname(dst_file)):
                os.makedirs(os.path.dirname(dst_file))

            template = self.jinja2env.get_template(item.template)
            with open(dst_file, 'w+') as dst:
                dst.write(template.render(item.tojinja2()))

        staticdst = os.path.join(self.destdir, 'static')
        if os.path.exists(staticdst):
            shutil.rmtree(staticdst)
        shutil.copytree(self.staticdir, staticdst)
        return self.jinja2env.globals.get('posts')
